from enum import Enum

from inoopa_utils.custom_types.addresses import Country


class EmployeeCategoryClass(Enum):
    class_0_employees = "0 employees"
    class_1_to_4_employees = "1 to 4 employees"
    class_5_to_9_employees = "5 to 9 employees"
    class_10_to_19_employees = "10 to 19 employees"
    class_20_to_49_employees = "20 to 49 employees"
    class_50_to_99_employees = "50 to 99 employees"
    class_100_to_199_employees = "100 to 199 employees"
    class_200_to_499_employees = "200 to 499 employees"
    class_500_to_999_employees = "500 to 999 employees"
    class_1000_to_9999999_employees = "1000 to 9999999 employees"
    not_found = "NOT FOUND"

    @classmethod
    def get_all_values(cls):
        """Return all possible values for this enum as a list of enum."""
        return [v for v in cls.__dict__.values() if isinstance(v, cls)]

    @classmethod
    def get_all_values_str(cls) -> list[str]:
        """Return all possible values for this enum as a list of str."""
        return [v.value for v in cls.__dict__.values() if isinstance(v, cls)]


class EntityType(Enum):
    legal_person = "legal person"
    natural_person = "natural person"

    @classmethod
    def get_all_values(cls):
        """Return all possible values for this enum as a list of enum."""
        return [v for v in cls.__dict__.values() if isinstance(v, cls)]

    @classmethod
    def get_all_values_str(cls) -> list[str]:
        """Return all possible values for this enum as a list of str."""
        return [v.value for v in cls.__dict__.values() if isinstance(v, cls)]


class LegalFormCategory(Enum):
    for_profit = "for-profit"
    non_profit = "non-profit"
    public = "public"

    @classmethod
    def get_all_values(cls):
        """Return all possible values for this enum as a list of enum."""
        return [v for v in cls.__dict__.values() if isinstance(v, cls)]

    @classmethod
    def get_all_values_str(cls) -> list[str]:
        """Return all possible values for this enum as a list of str."""
        return [v.value for v in cls.__dict__.values() if isinstance(v, cls)]


class LegalFormBe(Enum):
    agricultural_company = "Agricultural company"
    autonomous_municipal_company = "Autonomous municipal company"
    autonomous_provincial_company = "Autonomous provincial company"
    brussels_capital_region_authority = "Brussels-Capital region authority"
    cpas_ocmw_association = "CPAS / OCMW Association"
    cities_and_municipalities = "Cities and municipalities"
    co_ownership_association = "Co-ownership association"
    common_law_company = "Common law company"
    company_or_association_without_legal_personality = "Company or association without legal personality"
    congolese_company = "Congolese company"
    cooperative_society = "Cooperative society"
    cooperative_society_old_regime = "Cooperative society (old regime)"
    cooperative_society_governed_by_public_law = "Cooperative society governed by public law"
    cooperative_society_governed_by_public_law_old_regime = "Cooperative society governed by public law (old regime)"
    cooperative_society_with_limited_liability = "Cooperative society with limited liability"
    cooperative_society_with_limited_liability_profit_share = (
        "Cooperative society with limited liability (profit share)"
    )
    cooperative_society_with_limited_liability_and_a_social_objective = (
        "Cooperative society with limited liability and a social objective"
    )
    cooperative_society_with_limited_liability_governed_by_public_law = (
        "Cooperative society with limited liability governed by public law"
    )
    cooperative_society_with_unlimited_liability = "Cooperative society with unlimited liability"
    cooperative_society_with_unlimited_liability_profit_share = (
        "Cooperative society with unlimited liability (profit share)"
    )
    cooperative_society_with_unlimited_liability_and_a_social_objective = (
        "Cooperative society with unlimited liability and a social objective"
    )
    economic_interest_grouping_with_a_social_objective = "Economic interest grouping with a social objective"
    economic_interest_grouping_with_registered_seat_in_belgium = (
        "Economic interest grouping with registered seat in Belgium"
    )
    europ_econ_assoc_wo_regseat_but_with_est_unit_in_belgium = (
        "Europ. Econ. assoc wo reg.seat but with est. unit in Belgium"
    )
    european_company_societas_europaea = "European company (Societas Europaea)"
    european_cooperative_society = "European cooperative society"
    european_economic_assoc_with_registered_seat_in_belgium = "European economic assoc with registered seat in Belgium"
    european_political_foundation = "European political foundation"
    european_political_party = "European political party"
    federal_public_planning_service = "Federal public planning service"
    federal_public_service = "Federal public service"
    flemish_region_and_flemish_community_authority = "Flemish region and Flemish community authority"
    foreign_ent_with_property_in_belgium_without_legal_pers = (
        "Foreign ent. with property in Belgium (without legal pers.)"
    )
    foreign_entity = "Foreign entity"
    foreign_entity_with_property_in_belgium_with_legal_personality = (
        "Foreign entity with property in Belgium (with legal personality)"
    )
    foreign_entity_without_belgian_establishment_unit_with_vat_representation = (
        "Foreign entity without Belgian establishment unit with VAT representation"
    )
    foreign_listed_company_without_belgian_establishment_unit = (
        "Foreign listed company without Belgian establishment unit"
    )
    foreign_or_international_public_organisations = "Foreign or international public organisations"
    french_community_authority = "French community authority"
    general_partnership = "General partnership"
    general_partnership_with_a_social_objective = "General partnership with a social objective"
    german_speaking_community_authority = "German-speaking community authority"
    health_fund_mutual_health_insurance_national_union_of_health_funds = (
        "Health fund / Mutual health insurance / National union of health funds"
    )
    hulpverleningszone = "Hulpverleningszone"
    intercommunal = "Intercommunal"
    international_non_profit_association = "International non-profit association"
    international_non_profit_association_governed_by_public_law = (
        "International non-profit association governed by public law"
    )
    international_scientific_organisation_under_belgian_law = "International scientific organisation under Belgian law"
    limited_partnership = "Limited partnership"
    limited_partnership_governed_by_public_law = "Limited partnership governed by public Law"
    local_police = "Local police"
    ministry_for_middle_class = "Ministry for Middle Class"
    ministry_of_economic_affairs = "Ministry of Economic Affairs"
    ministry_of_foreign_affairs = "Ministry of Foreign Affairs"
    ministry_of_home_affairs = "Ministry of Home Affairs"
    ministry_of_justice = "Ministry of Justice"
    ministry_of_the_brussels_capital_region = "Ministry of the Brussels-Capital Region"
    ministry_of_the_flemish_community = "Ministry of the Flemish Community"
    ministry_of_the_french_community = "Ministry of the French Community"
    ministry_of_the_walloon_region = "Ministry of the Walloon Region"
    miscellaneous = "Miscellaneous"
    miscellaneous_without_legal_personality = "Miscellaneous without legal personality"
    non_profit_institution = "Non-profit institution"
    non_profit_organisation = "Non-profit organisation"
    ordinary_limited_partnership = "Ordinary limited partnership"
    ordinary_limited_partnership_with_a_social_objective = "Ordinary limited partnership with a social objective"
    organis_regist_with_the_public_admin_pensions_finance = "Organis. regist. with the public admin. Pensions (Finance)"
    organisations_registered_with_the_onp = "Organisations registered with the O.N.P"
    other_federal_services = "Other federal services"
    other_institution_with_a_social_objective_public = "Other institution with a social objective (public)"
    other_legal_form = "Other legal form"
    other_private_organisation_with_legal_personality = "Other private organisation with legal personality"
    partnership_limited_by_shares = "Partnership limited by shares"
    partnership_limited_by_shares_with_a_social_objective = "Partnership limited by shares with a social objective"
    pawnshop = "Pawnshop"
    pension_scheme_organisation = "Pension scheme organisation"
    polders_and_water_boards = "Polders and water boards"
    private_foreign_association_with_establishment_in_belgium = (
        "Private foreign association with establishment in Belgium"
    )
    private_foundation = "Private foundation"
    private_limited_company = "Private limited company"
    private_limited_company_governed_by_public_law = "Private limited company governed by public law"
    private_limited_liability_company = "Private limited liability company"
    private_limited_liability_company_with_a_social_objective = (
        "Private limited liability company with a social objective"
    )
    private_mutual_insurance_fund = "Private mutual insurance fund"
    professional_corporations_orders = "Professional corporations - Orders"
    professional_union = "Professional union"
    project_association = "Project association"
    provincial_authority = "Provincial authority"
    pubic_social_action_centre = "Pubic social action centre"
    public_institution = "Public institution"
    public_limited_company = "Public limited company"
    public_limited_company_with_a_social_objective = "Public limited company with a social objective"
    public_non_profit_association = "Public non-profit association"
    public_utility_foundation = "Public utility foundation"
    public_utility_institution = "Public utility institution"
    representative_association_flemish_region = "Representative association (Flemish region)"
    service_provider_association_flemish_region = "Service provider association (Flemish region)"
    state_province_region_community = "State, Province, Region, Community"
    temporary_association = "Temporary association"
    the_services_of_the_prime_minister = "The services of the Prime Minister"
    trade_union = "Trade union"
    unkown_legal_form_nsso = "Unkown legal form (NSSO)"
    vat_group = "VAT-group"
    walloon_region_authorit = "Walloon region authorit"

    @classmethod
    def get_all_values(cls):
        """Return all possible values for this enum as a list of enum."""
        return [v for v in cls.__dict__.values() if isinstance(v, cls)]

    @classmethod
    def get_all_values_str(cls) -> list[str]:
        """Return all possible values for this enum as a list of str."""
        return [v.value for v in cls.__dict__.values() if isinstance(v, cls)]


class LegalFormNl(Enum):
    """"""

    @classmethod
    def get_all_values(cls):
        """Return all possible values for this enum as a list of enum."""
        return [v for v in cls.__dict__.values() if isinstance(v, cls)]

    @classmethod
    def get_all_values_str(cls) -> list[str]:
        """Return all possible values for this enum as a list of str."""
        return [v.value for v in cls.__dict__.values() if isinstance(v, cls)]

    Sole_Proprietorship = "Sole Proprietorship"
    Civil_law_partnership = "Civil-law partnership"
    Partnership_firm = "Partnership firm"
    Limited_partnership = "Limited partnership"
    Private_limited_company = "Private limited company"
    Limited_company = "Limited company"
    Cooperative_association = "Cooperative association"
    Association = "Association"
    Foundation = "Foundation"
    Mutual_insurance_association = "Mutual insurance association"
    Foreign_legal_form_branch_of_a_foreign_company = "Foreign legal form (branch of a foreign company)"
    Shipping_company = "Shipping company"
    European_economical_cooperative_organisation = "European economical cooperative organisation"
    Church = "Church"
    Legal_entity_in_formation = "Legal entity in formation"
    Public_partnership = "Public partnership"
    Eropean_limited_company = "Eropean limited company"
    Owners_association = "Owners' association"
    Legal_entity_under_public_law = "Legal entity under public law"
    Legal_entity_under_private_law = "Legal entity under private law"
    Body_governed_by_public_law = "Body governed by public law"
    European_limited_company = "European limited company"


class LegalFormFr(Enum):
    def __new__(cls, value):
        raise NotImplementedError("FR legal forms not implemented yet")

    @classmethod
    def get_all_values(cls):
        """Return all possible values for this enum as a list of enum."""
        return [v for v in cls.__dict__.values() if isinstance(v, cls)]

    @classmethod
    def get_all_values_str(cls) -> list[str]:
        """Return all possible values for this enum as a list of str."""
        return [v.value for v in cls.__dict__.values() if isinstance(v, cls)]


def get_all_legal_forms(countries: list[Country] | list[str]) -> list[LegalFormBe | LegalFormFr | LegalFormNl]:
    """Get all legal forms for a list of countries. Countries can be a list of Country enums or a list of str."""
    all_legal_forms = []
    if not countries:
        return all_legal_forms

    if isinstance(countries[0], str):
        countries = [Country(country) for country in countries]

    for country in countries:
        if country == Country.belgium:
            all_legal_forms.extend(LegalFormBe.get_all_values())
        elif country == Country.france:
            all_legal_forms.extend(LegalFormFr.get_all_values())
        elif country == Country.netherlands:
            all_legal_forms.extend(LegalFormNl.get_all_values())
        else:
            raise ValueError(f"Unknown country: `{country}`")

    return all_legal_forms


def get_all_legal_forms_str(countries: list[Country] | list[str]) -> list[str]:
    """Get all legal forms for a list of countries. Countries can be a list of Country enums or a list of str."""
    all_legal_forms = []
    if not countries:
        return all_legal_forms

    if isinstance(countries[0], str):
        countries = [Country(country) for country in countries]

    for country in countries:
        if country == Country.belgium:
            all_legal_forms.extend(LegalFormBe.get_all_values_str())
        elif country == Country.france:
            all_legal_forms.extend(LegalFormFr.get_all_values_str())
        elif country == Country.netherlands:
            all_legal_forms.extend(LegalFormNl.get_all_values_str())
        else:
            raise ValueError(f"Unknown country: `{country}`")

    return all_legal_forms



from datetime import datetime
from typing import Literal

from pydantic import BaseModel

from inoopa_utils.custom_types.addresses import (
    Country,
    RegionBe,
    ProvinceBe,
    RegionFr,
    ProvinceFr,
    RegionNl,
    ProvinceNl,
)
from inoopa_utils.custom_types.legal_filters import EmployeeCategoryClass, LegalFormBe, LegalFormCategory, EntityType


MongoFilters = dict[str, dict[str, dict[str, int | str | list[str]]]]

AdditionalFields = Literal["email", "phone", "website", "nace_codes", "social_medias"]


class LeadGenerationCompanyFilters(BaseModel):
    """
    Represents the filters used to generate the leads.

    This is a pydantic model, so it can be used in the apis as a parameter.
    """

    additional_fields: list[AdditionalFields] | None = ["email", "phone", "website", "nace_codes", "social_medias"]

    countries: list[Country] | None = None
    regions: list[RegionBe | RegionFr | RegionNl] | None = None
    provinces: list[ProvinceBe | ProvinceFr | ProvinceNl] | None = None
    zipcodes: list[str] | None = None
    declared_best_nace_codes: list[str] | None = None
    declared_best_nace_codes_regex: str | None = None
    inoopa_best_nace_codes: list[str] | None = None
    inoopa_best_nace_codes_regex: str | None = None

    minimum_number_of_estanlishments: int | None = None
    maximum_number_of_estanlishments: int | None = None
    employee_categories: list[EmployeeCategoryClass] | None = None
    created_before: datetime | None = None
    created_after: datetime | None = None
    include_decision_makers: bool = False

    max_results: int | None = None
    excluded_companies: list[str] | None = None

    legal_form_Categories: list[LegalFormCategory] | None = None
    legal_forms: list[LegalFormBe] | None = None
    entity_types: list[EntityType] | None = None

    def to_mongo_filters(
        self,
        only_include_companies_with_phone_no_do_not_call_me: bool = False,
        only_companies_with_attributed_phone: bool = False,
        only_include_companies_with_email: bool = False,
        only_include_companies_with_website: bool = False,
        only_include_companies_with_nace_codes: bool = False,
        only_include_active_companies: bool = True,
    ) -> MongoFilters:
        """
        Generate a MongoDB filter to search for companies based on the LeadGenerationCompanyFilters.

        can be used as a filter parameter in the MongoDB Collection.find() method.
        """
        filters = {}
        if self.countries:
            filters["country"] = {"$in": [c.value for c in self.countries]}
        if self.legal_forms:
            filters["legal_form"] = {"$in": [form.value for form in self.legal_forms]}
        if self.employee_categories:
            filters["employee_category_formatted"] = {"$in": [c.value for c in self.employee_categories]}
        if self.regions:
            regions = [r.value if r.value != RegionBe.not_found.value else None for r in self.regions]
            filters["address.region"] = {"$in": regions}
        if self.provinces:
            provinces = [p.value if p.value != ProvinceBe.not_found.value else None for p in self.provinces]
            filters["address.province"] = {"$in": provinces}
        if self.zipcodes:
            zipcodes = [z if z != "NOT FOUND" else None for z in self.zipcodes]
            filters["address.postal_code"] = {"$in": zipcodes}

        if self.minimum_number_of_estanlishments and self.maximum_number_of_estanlishments:
            filters["number_of_establishments"] = {
                "$gte": self.minimum_number_of_estanlishments,
                "$lte": self.maximum_number_of_estanlishments,
            }
        elif self.minimum_number_of_estanlishments:
            filters["number_of_establishments"] = {"$gte": self.minimum_number_of_estanlishments}
        elif self.maximum_number_of_estanlishments:
            filters["number_of_establishments"] = {"$lte": self.maximum_number_of_estanlishments}

        if self.declared_best_nace_codes:
            filters["$or"] = [
                {"nace_codes": {"$elemMatch": {"number": {"$in": self.declared_best_nace_codes}}}},
                {
                    "establishments": {
                        "$elemMatch": {"nace_codes": {"$elemMatch": {"number": {"$in": self.declared_best_nace_codes}}}}
                    }
                },
            ]
        if self.declared_best_nace_codes_regex:
            nace_section_codes = []
            naces_codes_regex = []
            for code_regex in self.declared_best_nace_codes_regex.split("|"):
                # if the regex is only one letter, it's a nace section code
                if len(code_regex) == 1:
                    nace_section_codes.append(code_regex)
                else:
                    # if the regex is more than one letter, it's a nace code regex
                    naces_codes_regex.append(code_regex)

            filters["$or"] = []
            if naces_codes_regex:
                filters["$or"].append(
                    {"nace_codes": {"$elemMatch": {"number": {"$regex": "|".join(naces_codes_regex)}}}}
                )
                filters["$or"].append(
                    {
                        "establishments": {
                            "$elemMatch": {
                                "nace_codes": {"$elemMatch": {"number": {"$regex": "|".join(naces_codes_regex)}}}
                            }
                        }
                    },
                )
            if nace_section_codes:
                filters["$or"].append({"nace_codes": {"$elemMatch": {"section_code": {"$in": nace_section_codes}}}})
                filters["$or"].append(
                    {
                        "establishments": {
                            "$elemMatch": {"nace_codes": {"$elemMatch": {"section_code": {"$in": nace_section_codes}}}}
                        }
                    },
                )

        if self.inoopa_best_nace_codes:
            filters["best_nace_codes.first_best_nace_code.number"] = {"$in": self.inoopa_best_nace_codes}
        if self.inoopa_best_nace_codes_regex:
            nace_codes = []
            nace_section_codes = []
            for code_regex in self.inoopa_best_nace_codes_regex.split("|"):
                # if the regex is only one letter, it's a nace section code
                if len(code_regex) == 1:
                    nace_section_codes.append(code_regex)
                else:
                    # if the regex is more than one letter, it's a nace code regex
                    nace_codes.append(code_regex)
            filters["$or"] = []
            if nace_codes:
                filters["$or"].append({"best_nace_codes.first_best_nace_code.number": {"$regex": "|".join(nace_codes)}})
            if nace_section_codes:
                filters["$or"].append(
                    {"best_nace_codes.first_best_nace_code.section_code": {"$in": nace_section_codes}}
                )

        if self.legal_form_Categories:
            filters["legal_form_type"] = {"$in": [c.value for c in self.legal_form_Categories]}
        if self.entity_types:
            filters["entity_type"] = {"$in": [e.value for e in self.entity_types]}
        if self.created_before and not self.created_after:
            filters["start_date"] = {"$lte": self.created_before}
        if self.created_after and not self.created_before:
            filters["start_date"] = {"$gte": self.created_after}
        if self.created_before and self.created_after:
            filters["start_date"] = {"$lte": self.created_before, "$gte": self.created_after}
        if self.excluded_companies:
            filters["_id"] = {"$nin": self.excluded_companies}
        if only_include_active_companies:
            filters["status"] = "Active"
            filters["legal_situation"] = "Normal situation"

        if only_include_companies_with_phone_no_do_not_call_me or only_companies_with_attributed_phone:
            filters["best_phone"] = {"$ne": None}
            filters["best_phone.phone"] = {"$nin": ["DO_NOT_CALL_ME", None]}
            if only_companies_with_attributed_phone:
                filters["best_phone.operator"] = {"$ne": None}
                filters["best_phone.operator_last_update"] = {"$ne": None, "$gt": datetime(2009, 12, 31)}

        if only_include_companies_with_email:
            filters["best_email"] = {"$ne": None}
        if only_include_companies_with_website:
            filters["best_website"] = {"$ne": None}
        if only_include_companies_with_nace_codes:
            filters["best_nace_codes"] = {"$ne": None}
        return filters

    def to_dict(self) -> dict:
        data_dict = self.model_dump()
        # Convert datetime objects to isoformat for json serialization
        if self.created_before:
            data_dict["created_before"] = _datetime_serializer(self.created_before)
        if self.created_after:
            data_dict["created_after"] = _datetime_serializer(self.created_after)
        return data_dict


class EnrichmentCompanyFilters(BaseModel):
    vats_to_enrich: list[str]
    country: Country = Country.belgium
    additional_fields: list[AdditionalFields] = ["email", "phone", "website", "nace_codes", "social_medias"]
    include_decision_makers: bool = False

    def to_mongo_filters(
        self,
        only_include_companies_with_phone_no_do_not_call_me: bool = False,
        only_include_companies_with_email: bool = False,
        only_include_companies_with_website: bool = False,
        only_include_companies_with_nace_codes: bool = False,
    ) -> MongoFilters:
        filters = {"country": self.country.value, "_id": {"$in": self.vats_to_enrich}}
        if only_include_companies_with_phone_no_do_not_call_me:
            filters["best_phone"] = {"$ne": None}
        if only_include_companies_with_email:
            filters["best_email"] = {"$ne": None}
        if only_include_companies_with_website:
            filters["best_website"] = {"$ne": None}
        if only_include_companies_with_nace_codes:
            filters["best_nace_codes"] = {"$ne": None}
        return filters

    def to_dict(self) -> dict:
        return self.model_dump()


class SemanticSearchCompanyFilters(BaseModel):
    countries: list[Country] = Country.get_all_values()
    regions: list[RegionBe | RegionFr | RegionNl] | None = None
    zipcodes: list[str] | None = None
    declared_best_nace_codes: list[str] | None = None
    declared_best_nace_codes_regex: str | None = None
    inoopa_best_nace_codes: list[str] | None = None
    inoopa_best_nace_codes_regex: str | None = None

    minimum_number_of_estanlishments: int | None = None
    maximum_number_of_estanlishments: int | None = None
    employee_categories: list[EmployeeCategoryClass] | None = None
    created_before: datetime | None = None
    created_after: datetime | None = None
    include_decision_makers: bool = False

    max_results: int | None = None
    excluded_companies: list[str] | None = None

    legal_form_types: list[LegalFormCategory] | None = None
    legal_forms: list[LegalFormBe] | None = None

    additional_fields: list[AdditionalFields] | None = None

    def to_dict(self) -> dict:
        data_dict = self.model_dump()
        # Convert datetime objects to isoformat for json serialization
        if self.created_before:
            data_dict["created_before"] = _datetime_serializer(self.created_before)
        if self.created_after:
            data_dict["created_after"] = _datetime_serializer(self.created_after)
        return data_dict


def _datetime_serializer(obj):
    if isinstance(obj, datetime):
        return obj.isoformat()
    raise TypeError(f"Type {type(obj)} not serializable")
